import { fetch } from './request'
import { times } from './tools'

// 高级队列, 先执行高级队列
// 高级队列一般放置, 影响页面显示及交互的请求
const tq = [];

// 再执行中级队列
// 中级队列一般放置, 与页面及交互无关的请求, 比如数据更新之类的
const mq = [];

// 最后执行低级队列
// 低级队列一般放置, 与业务无关的请求, 比如埋点之类的
const lq = [];

// 汇总队列
const qs = [tq, mq, lq];

// 当前 request loop 是否执行中
let isProcessing = false

// 执行一个队列
function execQ(qt) {
  const qLen = qs[qt].length
  if (process.env.NODE_ENV === 'development') {
    console.log(`[request-queue] [${qt}]: 当前队列长度 ${qLen}`, qs[qt])
  }

  if (qLen === 0) {
    return Promise.resolve()
  }

  return new Promise((resolve) => {
    times(qLen).forEach(() => {

      const current = qs[qt].shift()
      const [options, success, fail] = current

      fetch(options).then((res) => {
        success(res)
      }).catch((err) => {
        fail(err)
      })

    })

    resolve()
  })
}

// 添加任务队列, 如果空闲状态则立即执行
function addToQs(qt, job) {
  qs[qt].push(job)

  if (!isProcessing) {
    startQs()
  }
}

async function startQs() {
  isProcessing = true

  await execQ(0)
  await execQ(1)
  await execQ(2)

  if (qs.reduce((s,q) => s + q.length, 0) > 0) {
    await startQs()
  }

  isProcessing = false
}

export function fetchQ(options, queueType) {
  return new Promise((resolve, reject) => {
    // 默认放置高级队列
    const qt = queueType || 0
    addToQs(qt, [options, resolve, reject])
  })
}
